// ---------------------------------------------------------------------------
// Module:		FU_SETUP.C
//
// Description:	FreeUp's Setup/Options routine & dialog procedures
//
// Last modified: 07/04/94
//
// (C) Copyright 1994 Thomas R. Grubbe, All Rights Reserved
// ---------------------------------------------------------------------------

#define STRICT
#include <windows.h>
#include <windowsx.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <dos.h>
#include <dir.h>
#include <ctl3d.h>
#include "freeup.h"
#include "fu_setup.h"
#include "fu_cmd.h"
#include "fu_util.h"


int SetUpFreeUp(void);
BOOL CALLBACK _export OptionsDlgProc (HWND, UINT, WPARAM, LPARAM);
BOOL CALLBACK _export DetailsDlgProc (HWND, UINT, WPARAM, LPARAM);
BOOL CALLBACK _export DirectoriesDlgProc (HWND, UINT, WPARAM, LPARAM);

LRESULT CALLBACK _export NewEditProc1(HWND, UINT, WPARAM, LPARAM);
LRESULT CALLBACK _export NewEditProc2(HWND, UINT, WPARAM, LPARAM);
LRESULT CALLBACK _export NewEditProc3(HWND, UINT, WPARAM, LPARAM);

WNDPROC OldEdit1;
WNDPROC OldEdit2;
WNDPROC OldEdit3;


int SetUpFreeUp(void)
{
	FARPROC DlgProc;
    HANDLE hLibrary;
	int nRet;
	char CurDir[128];

	InitFreeUp();
	getcwd(CurDir, 128);
	LogPath(StartUpDir);

	DlgProc = MakeProcInstance((FARPROC)OptionsDlgProc, ghInstance);
	nRet = DialogBox(ghInstance, "OptionsDlg", ghwndMain, (DLGPROC)DlgProc);
	FreeProcInstance(DlgProc);

	LogPath(CurDir);
	return nRet;
}


#pragma argsused
BOOL CALLBACK _export OptionsDlgProc (HWND hDlg, UINT msg,
											WPARAM wParam, LPARAM lParam)
{
	HFONT hFont;
    LOGFONT lFont;

	switch (msg)
	{
		case WM_INITDIALOG:
		{
			hFont = (HFONT)NULL;
			if ((hFont = (HFONT)SendMessage(hDlg, WM_GETFONT, 0, 0L)) != 0)
            {
				if (GetObject(hFont, sizeof(LOGFONT), (LPSTR)&lFont))
                {
					lFont.lfWeight = FW_NORMAL;
					if ((hFont = CreateFontIndirect((LPLOGFONT)&lFont)) != 0)	{
						SendMessage(GetDlgItem(hDlg, IDC_FILEDELPRMPT), WM_SETFONT, (WPARAM)hFont, FALSE);
						SendMessage(GetDlgItem(hDlg, IDC_DIRDELPRMPT), WM_SETFONT, (WPARAM)hFont, FALSE);
						SendMessage(GetDlgItem(hDlg, IDC_OVRWRITEPRMPT), WM_SETFONT, (WPARAM)hFont, FALSE);
						SendMessage(GetDlgItem(hDlg, IDC_SAVEWNDWPOS), WM_SETFONT, (WPARAM)hFont, FALSE);
						SendMessage(GetDlgItem(hDlg, IDC_LOWERCASES), WM_SETFONT, (WPARAM)hFont, FALSE);
						SendMessage(GetDlgItem(hDlg, IDC_SHOWTBAR), WM_SETFONT, (WPARAM)hFont, FALSE);
						SendMessage(GetDlgItem(hDlg, IDC_SHOWSBAR), WM_SETFONT, (WPARAM)hFont, FALSE);
						SendMessage(GetDlgItem(hDlg, IDC_DIRSIZEINFO), WM_SETFONT, (WPARAM)hFont, FALSE);
						SendMessage(GetDlgItem(hDlg, IDC_CUSTNAMES), WM_SETFONT, (WPARAM)hFont, FALSE);
						SendMessage(GetDlgItem(hDlg, IDC_FOLDERICONS), WM_SETFONT, (WPARAM)hFont, FALSE);
						SendMessage(GetDlgItem(hDlg, IDC_FILEICONS), WM_SETFONT, (WPARAM)hFont, FALSE);
						SendMessage(GetDlgItem(hDlg, IDC_FILENAMESONLY), WM_SETFONT, (WPARAM)hFont, FALSE);
						SendMessage(GetDlgItem(hDlg, IDC_PARTIALDETAILS), WM_SETFONT, (WPARAM)hFont, FALSE);
						SendMessage(GetDlgItem(hDlg, IDC_SHOWALLDETAILS), WM_SETFONT, (WPARAM)hFont, FALSE);
                    }
				}
			}

			if (LowerCase)
				CheckDlgButton(hDlg, IDC_LOWERCASES, IsDlgButtonChecked(hDlg, IDC_LOWERCASES) ? 0 : 1);
			if (DirSizeInfo)
				CheckDlgButton(hDlg, IDC_DIRSIZEINFO, IsDlgButtonChecked(hDlg, IDC_DIRSIZEINFO) ? 0 : 1);
			if (UseCustomPathNames)
				CheckDlgButton(hDlg, IDC_CUSTNAMES, IsDlgButtonChecked(hDlg, IDC_CUSTNAMES) ? 0 : 1);
			// ...
			return TRUE;
		}

		case WM_CTLCOLOR:
		{
			return (BOOL)Ctl3dCtlColorEx(msg, wParam, lParam);
		}

		case WM_COMMAND:
            switch (wParam)
			{
				case IDC_FILEDELPRMPT:
				case IDC_DIRDELPRMPT:
				case IDC_OVRWRITEPRMPT:
				case IDC_SAVEWNDWPOS:
				case IDC_LOWERCASES:
				case IDC_SHOWTBAR:
				case IDC_SHOWSBAR:
				case IDC_DIRSIZEINFO:
				case IDC_CUSTNAMES:
				case IDC_FOLDERICONS:
				case IDC_FILEICONS:
					CheckDlgButton(hDlg, wParam,
						IsDlgButtonChecked(hDlg, wParam) ? 0 : 1);
					return TRUE;
				case IDOK:
					LowerCase = IsDlgButtonChecked(hDlg, IDC_LOWERCASES) ? 1 : 0;
					DirSizeInfo = IsDlgButtonChecked(hDlg, IDC_DIRSIZEINFO) ? 1 : 0;
                    UseCustomPathNames = IsDlgButtonChecked(hDlg, IDC_CUSTNAMES) ? 1 : 0;
                    // ...
					DeInitFreeUp();
                    DeleteObject(hFont);
					EndDialog(hDlg, IDOK);
                    return TRUE;
				case IDCANCEL:
					SendMessage(hDlg, WM_CLOSE, wParam, 0L);
					return TRUE;
				case IDC_DETAILSBTN:
				{
					FARPROC DetailsProc = MakeProcInstance((FARPROC)DetailsDlgProc, ghInstance);
					DialogBox(ghInstance, "DetailsDlg", hDlg, (DLGPROC)DetailsProc);
					FreeProcInstance(DetailsProc);
					return TRUE;
                }
				case IDC_DIRECTORIESBTN:
                {
					FARPROC DirProc = MakeProcInstance((FARPROC)DirectoriesDlgProc, ghInstance);
					DialogBox(ghInstance, "DirectoriesDlg", hDlg, (DLGPROC)DirProc);
					FreeProcInstance(DirProc);
					return TRUE;
                }
            }
            break ;
		case WM_CLOSE:
			DeleteObject(hFont);
        	EndDialog(hDlg, IDCANCEL);
			return TRUE;
	}
	return FALSE;
}

#pragma argsused
BOOL CALLBACK _export DetailsDlgProc (HWND hDlg, UINT msg,
									  WPARAM wParam, LPARAM lParam)
{
	HFONT hFont;
    LOGFONT lFont;

	switch (msg)
	{
		case WM_INITDIALOG:
		{
			hFont = (HFONT)NULL;
			if ((hFont = (HFONT)SendMessage(hDlg, WM_GETFONT, 0, 0L)) != 0)
            {
				if (GetObject(hFont, sizeof(LOGFONT), (LPSTR)&lFont))
                {
					lFont.lfWeight = FW_NORMAL;
					if ((hFont = CreateFontIndirect((LPLOGFONT)&lFont)) != 0)	{
						SendMessage(GetDlgItem(hDlg, IDC_SIZEBTN), WM_SETFONT, (WPARAM)hFont, FALSE);
						SendMessage(GetDlgItem(hDlg, IDC_DATEBTN), WM_SETFONT, (WPARAM)hFont, FALSE);
						SendMessage(GetDlgItem(hDlg, IDC_TIMEBTN), WM_SETFONT, (WPARAM)hFont, FALSE);
						SendMessage(GetDlgItem(hDlg, IDC_ATTRBTN), WM_SETFONT, (WPARAM)hFont, FALSE);
                    }
				}
			}
			return TRUE;
		}

		case WM_CTLCOLOR:
		{
			return (BOOL)Ctl3dCtlColorEx(msg, wParam, lParam);
		}

		case WM_COMMAND:
            switch (wParam)
			{
				case IDOK:
                	DeleteObject(hFont);
					EndDialog(hDlg, IDOK);
                    return TRUE;
				case IDCANCEL:
					SendMessage(hDlg, WM_CLOSE, wParam, 0L);
					return TRUE;
            }
            break ;
		case WM_CLOSE:
        	DeleteObject(hFont);
        	EndDialog(hDlg, IDCANCEL);
			return TRUE;
	}
	return FALSE;
}
#pragma argsused
BOOL CALLBACK _export DirectoriesDlgProc (HWND hDlg, UINT msg,
										  WPARAM wParam, LPARAM lParam)
{
	static HFONT hFont;
	static LOGFONT lFont;
	static HWND hEdit1, hEdit2, hEdit3;
	static FARPROC lpfnEdit1, lpfnEdit2, lpfnEdit3;

	switch (msg)
	{
		case WM_INITDIALOG:
		{
			hFont = (HFONT)NULL;
			if ((hFont = (HFONT)SendMessage(hDlg, WM_GETFONT, 0, 0L)) != 0)
            {
				if (GetObject(hFont, sizeof(LOGFONT), (LPSTR)&lFont))
                {
					lFont.lfWeight = FW_NORMAL;
					if ((hFont = CreateFontIndirect((LPLOGFONT)&lFont)) != 0)	{
						SendMessage(GetDlgItem(hDlg, IDC_STARTUPPRMPT), WM_SETFONT, (WPARAM)hFont, FALSE);
						SendMessage(GetDlgItem(hDlg, IDC_EDITORPRMPT), WM_SETFONT, (WPARAM)hFont, FALSE);
						SendMessage(GetDlgItem(hDlg, IDC_VIEWERPRMPT1), WM_SETFONT, (WPARAM)hFont, FALSE);
						SendMessage(GetDlgItem(hDlg, IDC_VIEWERPRMPT2), WM_SETFONT, (WPARAM)hFont, FALSE);
                    }
				}
			}

			hEdit1 = GetDlgItem(hDlg, IDC_STARTUPEDIT);
			hEdit2 = GetDlgItem(hDlg, IDC_EDITOREDIT);
			hEdit3 = GetDlgItem(hDlg, IDC_VIEWEREDIT);

			lpfnEdit1 = MakeProcInstance((FARPROC)NewEditProc1, ghInstance);
            OldEdit1  = (WNDPROC)SetWindowLong(hEdit1, GWL_WNDPROC, (LONG)lpfnEdit1);
			lpfnEdit2 = MakeProcInstance((FARPROC)NewEditProc2, ghInstance);
			OldEdit2  = (WNDPROC)SetWindowLong(hEdit2, GWL_WNDPROC, (LONG)lpfnEdit2);
			lpfnEdit3 = MakeProcInstance((FARPROC)NewEditProc3, ghInstance);
			OldEdit3  = (WNDPROC)SetWindowLong(hEdit3, GWL_WNDPROC, (LONG)lpfnEdit3);

			SetWindowText(hEdit1, (LPCSTR)StartUpDir);
			SetWindowText(hEdit2,  (LPCSTR)DefaultEditor);
			SetWindowText(hEdit3,  (LPCSTR)DefaultViewer);

			return TRUE;
		}

		case WM_CTLCOLOR:
		{
			return (BOOL)Ctl3dCtlColorEx(msg, wParam, lParam);
		}

		case WM_COMMAND:
            switch (wParam)
			{
				case IDOK:
					GetWindowText(GetDlgItem(hDlg, IDC_STARTUPEDIT), StartUpDir, sizeof(StartUpDir));
					GetWindowText(GetDlgItem(hDlg, IDC_EDITOREDIT), DefaultEditor, sizeof(DefaultEditor));
					GetWindowText(GetDlgItem(hDlg, IDC_VIEWEREDIT), DefaultViewer, sizeof(DefaultViewer));
                	DeleteObject(hFont);
					EndDialog(hDlg, IDOK);
                    return TRUE;
				case IDCANCEL:
					SendMessage(hDlg, WM_CLOSE, wParam, 0L);
					return TRUE;
            }
            break ;
		case WM_CLOSE:
			DeleteObject(hFont);

			SetWindowLong(hEdit3, GWL_WNDPROC, (LONG)OldEdit3);
			FreeProcInstance(lpfnEdit3);
			SetWindowLong(hEdit2, GWL_WNDPROC, (LONG)OldEdit2);
			FreeProcInstance(lpfnEdit2);
			SetWindowLong(hEdit1, GWL_WNDPROC, (LONG)OldEdit1);
			FreeProcInstance(lpfnEdit1);
        	EndDialog(hDlg, IDCANCEL);
			return TRUE;
	}
	return FALSE;
}

LRESULT CALLBACK _export NewEditProc1(HWND hwnd, UINT msg,
									 WPARAM wParam, LPARAM lParam)
{
	switch (msg)
	{
		case WM_SETFOCUS:
			PostMessage(hwnd, EM_SETSEL, TRUE, MAKELONG(0, -1));
			break;
	}
    return CallWindowProc(OldEdit1, hwnd, msg, wParam, lParam);
}
LRESULT CALLBACK _export NewEditProc2(HWND hwnd, UINT msg,
									 WPARAM wParam, LPARAM lParam)
{
	switch (msg)
	{
		case WM_SETFOCUS:
			PostMessage(hwnd, EM_SETSEL, TRUE, MAKELONG(0, -1));
			break;
	}
    return CallWindowProc(OldEdit2, hwnd, msg, wParam, lParam);
}
LRESULT CALLBACK _export NewEditProc3(HWND hwnd, UINT msg,
									 WPARAM wParam, LPARAM lParam)
{
	switch (msg)
	{
		case WM_SETFOCUS:
			PostMessage(hwnd, EM_SETSEL, TRUE, MAKELONG(0, -1));
			break;
	}
	return CallWindowProc(OldEdit3, hwnd, msg, wParam, lParam);
}